
#include <stdio.h>
#include <stdlib.h>

#include "format.h"
#include "template.h"

typedef enum {

 INVALID, IDENT, NUMBER, STRING, CHAR, SEMICOLON, COMMA,
 LPAR, RPAR, PLUS, MINUS, ASTERISK, DIVIDE, EQUALS,
 BEGIN, END, ASSIGNMENT, DOT, DEREFERENCE,
 LESS, LESSOREQUAL, GREATER, GREATEROREQUAL, LBRACKET, RBRACKET,
 NEGATION, QUESTION, COLON, DOUBLECOLON, TILDE, AND, OR, LOGICALAND,
 LOGICALOR, XOR, TYPEDEF, STRUCT, CLASS, ENUM, CONST,
 UNSIGNED, STATIC, EXTERN, INLINE, _CDECL,
 INCREMENT, DECREMENT, PLUSASSIGNMENT, MINUSASSIGNMENT,
 MULTASSIGNMENT, DIVASSIGNMENT, ANDASSIGNMENT, ORASSIGNMENT,
 BREAK, CONTINUE,
 FOR, WHILE, DO, IF, ELSE, SWITCH, CASE, DEFAULT, RETURN,
 DIRECTIVE, COMMENT, EOI,

} Token;

const char* TokenNames[] = {

 "INVALID", "IDENT", "NUMBER" , "STRING", "CHAR", "SEMICOLON", "COMMA",
 "LPAR", "RPAR", "PLUS", "MINUS", "ASTERISK",  "DIVIDE", "EQUALS",
 "BEGIN", "END", "ASSIGNMENT", "DOT", "DEREFERENCE", 
 "LESS", "LESSOREQUAL", "GREATER", "GREATEROREQUAL", "LBRACKET", "RBRACKET",
 "NEGATION", "QUESTION", "COLON", "DOUBLECOLON", "TILDE", "AND", "OR", "LOGICALAND",
 "LOGICALOR", "XOR", "TYPEDEF", "STRUCT", "CLASS", "ENUM", "CONST",
 "UNSIGNED", "STATIC", "EXTERN", "INLINE", "_CDECL",
 "INCREMENT", "DECREMENT", "PLUSASSIGNMENT", "MINUSASSIGNMENT",
 "MULTASSIGNMENT", "DIVASSIGNMENT", "ANDASSIGNMENT", "ORASSIGNMENT",
 "BREAK", "CONTINUE", 
 "FOR", "WHILE", "DO", "IF", "ELSE", "SWITCH", "CASE", "DEFAULT", "RETURN",
 "DIRECTIVE", "COMMENT", "EOI"

};
const int TokenCount = sizeof(TokenNames)/sizeof(char*);

typedef struct { const char* keyword; Token token; } Keyword;
const Keyword Keywords[] = {
  "typedef"  , TYPEDEF,
  "struct"   , STRUCT,
  "class"    , CLASS,
  "enum"     , ENUM,
  "const"    , CONST,
  "unsigned" , UNSIGNED,
  "static"   , STATIC,
  "extern"   , EXTERN,
  "inline"   , INLINE,
  "for"      , FOR,
  "while"    , WHILE,
  "do"       , DO,
  "if"       , IF,
  "else"     , ELSE,
  "switch"   , SWITCH,
  "case"     , CASE,
  "default"  , DEFAULT,
  "return"   , RETURN,
  "break"    , BREAK,
  "continue" , CONTINUE,
  "__cdecl"  , _CDECL,
};
const int KeywordsCount = sizeof(Keywords)/sizeof(Keyword);

class CPosition {
public:
  int line, character;
  CPosition() {
    line = character = -1;
  }
};

class CFileBuffer {
private:
  CList<CStr> lines;
  CStr line;
public:
  void Add(int ch) {
    if (ch=='\n' || ch=='\r' || ch==EOF) {
      lines.Add(line);
      line = "";
    }
    else {
      line += ch;
    }
  }
  void Clean(CPosition start, CPosition stop) {
    if (start.line==stop.line) {
      lines[start.line].Set(start.character, stop.character, '');
    }
    else {
      lines[start.line].Set(start.character, lines[start.line].Length(), '');
      for (int line = start.line+1; line<stop.line; line++) {
        lines[line].Set(0, lines[line].Length(), '');
      }
      lines[stop.line].Set(0, stop.character, '');
    }
  }
  void Output(FILE *output) {
    for (int line=0; line<lines.Count(); line++) {
      lines[line].Remove('');
      fprintf(output,lines[line]+"\n");
    }
  }
};

class CLexicalAnalyzer {
private:
  FILE *input;
  Token next;
  int ch;
  CPosition actualpos, lastcharpos, tokenpos, lasttokenpos;
  CStr last, content;
  bool ignored[TokenCount];
public:
  CFileBuffer buffer;
  int Line() { return actualpos.line; }
  int Character() { return actualpos.character; }
private:
  void Error(const char *msg) {
    printf("Lexical error at line %d, character %d: %s\n", Line()+1, Character()+1, msg);
    exit(1);
  }
  bool IsBlank() {
    return (ch==' ' || ch=='\t' || ch=='\n' || ch=='\r');
  }
  bool IsNewLine() {
    return (ch=='\n' || ch=='\r');
  }
  bool IsAlpha() {
    return ('A'<=ch && ch<='Z')||('a'<=ch && ch<='z')||(ch=='_');
  }
  bool IsNumeric() {
    return ('0'<=ch && ch<='9');
  }
  bool IsSpecialNumeric() {
    return (ch=='x' || ch=='X' || ch=='u' || ch=='U' || ch=='l' || ch=='L');
  }
  bool IsAlphaNumeric() {
    return IsAlpha() || IsNumeric();
  }
  void ReadChar() {
    if (ch==EOF) Error("Unexpected EOF");
    ch = getc(input);
    lastcharpos = actualpos;
    if (IsNewLine()) { actualpos.line++; actualpos.character = 0; }
    else actualpos.character++;
    buffer.Add(ch);
  }
  void ParseToken() {
    while (IsBlank()) ReadChar();
    tokenpos = lastcharpos;
    if (IsAlpha()) {
      content = "";
      do {
        content += ch;
        ReadChar();
      } while (IsAlphaNumeric());
      for (int i=0; i<KeywordsCount; i++) {
        if (content==Keywords[i].keyword) {
          next = Keywords[i].token;
          return;
        }
      }
      next = IDENT;
      return;
    }
    else if (IsNumeric()) {
      content = "";
      do {
        content += ch;
        ReadChar();
      } while (IsNumeric()||IsSpecialNumeric());
      next = NUMBER;
      return;
    }
    else if (ch=='"') {
      content = "";
      while(true) {
        ReadChar();
        if (ch=='\\') ReadChar();
        else if (ch=='\"') break;
        content += ch;
      }
      next = STRING;
    }
    else if (ch=='\'') {
      content = "";
      while(true) {
        ReadChar();
        if (ch=='\\') ReadChar();
        else if (ch=='\'') break;
        content += ch;
      }
      next = CHAR;
    }
    else if (ch=='#') {
      while (!IsNewLine()) ReadChar();
      next = DIRECTIVE;
    }
    else if (ch=='/') {
      ReadChar();
      if (ch=='/') {
        while (!IsNewLine()) ReadChar();
        next = COMMENT;
      }
      else if (ch=='*') {
        while (true) {
          ReadChar();
          if (ch=='*') {
            ReadChar();
            if (ch=='/') {
              ReadChar();
              break;
            }
          }
        }
        next = COMMENT;
      }
      else if (ch=='=') {
        next = DIVASSIGNMENT;
      }
      else {
        next = DIVIDE;
        return;
      }
    }
    else if (ch=='=') {
      ReadChar();
      if (ch=='=') {
        next = EQUALS;
      }
      else {
        next = ASSIGNMENT;
        return;
      }
    }
    else if (ch=='-') {
      ReadChar();
      if (ch=='>') {
        next = DEREFERENCE;
      }
      else if (ch=='-') {
        next = DECREMENT;
      }
      else if (ch=='=') {
        next = MINUSASSIGNMENT;
      }
      else {
        next = MINUS;
        return;
      }
    }
    else if (ch=='+') {
      ReadChar();
      if (ch=='+') {
        next = INCREMENT;
      }
      else if (ch=='=') {
        next = PLUSASSIGNMENT;
      }
      else {
        next = PLUS;
        return;
      }
    }
    else if (ch=='*') {
      ReadChar();
      if (ch=='=') {
        next = MULTASSIGNMENT;
      }
      else {
        next = ASTERISK;
        return;
      }
    }
    else if (ch=='<') {
      ReadChar();
      if (ch=='=') {
        next = LESSOREQUAL;
      }
      else {
        next = LESS;
        return;
      }
    }
    else if (ch=='>') {
      ReadChar();
      if (ch=='=') {
        next = GREATEROREQUAL;
      }
      else {
        next = GREATER;
        return;
      }
    }
    else if (ch=='&') {
      ReadChar();
      if (ch=='&') {
        next = LOGICALAND;
      }
      else if (ch=='=') {
        next = ANDASSIGNMENT;
      }
      else {
        next = AND;
        return;
      }
    }
    else if (ch=='|') {
      ReadChar();
      if (ch=='|') {
        next = LOGICALOR;
      }
      else if (ch=='=') {
        next = ORASSIGNMENT;
      }
      else {
        next = OR;
        return;
      }
    }
    else if (ch==':') {
      ReadChar();
      if (ch==':') {
        next = DOUBLECOLON;
      }
      else {
        next = COLON;
        return;
      }
    }
    else if (ch==EOF) {
      if (next==EOI) Error("Unexpected EOI");
      next = EOI;
      return;
    }
    else if (ch==';') next = SEMICOLON;
    else if (ch==',') next = COMMA;
    else if (ch=='(') next = LPAR;
    else if (ch==')') next = RPAR;
    else if (ch=='[') next = LBRACKET;
    else if (ch==']') next = RBRACKET;
    else if (ch=='{') next = BEGIN;
    else if (ch=='}') next = END;
    else if (ch=='.') next = DOT;
    else if (ch=='!') next = NEGATION;
    else if (ch=='?') next = QUESTION;
    else if (ch=='~') next = TILDE;
    else if (ch=='^') next = XOR;
    else Error("Unexpected character");
    ReadChar();
  }
  void Parse() {
    do {
      ParseToken();
    } while(ignored[next]);
  }
public:
  CLexicalAnalyzer(FILE *input) {
    this->input = input;
    memset(ignored,0,sizeof(ignored));
    actualpos.line = actualpos.character = 0;
  }
  void Init() {
    ReadChar();
    Parse();
  }
  Token Next() {
    return next;
  }
  CStr Content() {
    return content;
  }
  CStr LastContent() {
    return last;
  }
  CPosition Position() {
    return tokenpos;
  }
  CPosition LastPosition() {
    return lasttokenpos;
  }
  void Read(Token token) {
    if (token!=next) {
      printf(".. expected %s, found %s\n",TokenNames[token],TokenNames[next]);
      Error("Unexpected token");
    };
//    printf("Token : %s\n",TokenNames[token]);
    last = content; content = "";
    lasttokenpos = tokenpos;
    Parse();
  }
  void SkipUntil(Token token) {
    while (Next()!=token) Read(Next());
  }
  void SkipPairs(Token left, Token right) {
    Read(left);
    while (Next()!=right) {
      if (Next()==left)
        SkipPairs(left,right);
      else
        Read(Next());
    }
    Read(right);
  }
  void Ignore(Token token) {
    ignored[token] = true;
  }
  void SkipUntil(Token token1, Token token2) {
    while (Next()!=token1 && Next()!=token2) Read(Next());
  }
};

class CVariable {
public:
  CStr name;
  bool used;
  bool protect;
  CPosition start, stop;
  CVariable(CStr name, CPosition start, CPosition stop, bool protect) {
    this->name    = name;
    this->used    = false;
    this->protect = protect;
    this->start   = start;
    this->stop    = stop;
  }
};

class CContext {
public:
  CList<CVariable*> variables;  // variables in the block
  CList<CContext*> contexts;    // child contexts
  CList<CStr> references;       // variables referenced in the block
  CList<CStr> calls;            // functions called in the block
  CContext *parent;
  CStr name;
  CPosition start, stop;
  bool executed;
// KONSTRUKCE STROMU
  CContext(CContext *parent = NULL) {
    this->name = "";
    this->parent = parent;
    if (parent) parent->contexts.Add(this);
    executed = false;
  }
  CContext(CContext *parent, CStr name, CPosition start) {
    this->name = name;
    this->start = start;
    this->parent = parent;
    if (parent) parent->contexts.Add(this);
    executed = false;
  }
  ~CContext() {
    int i;
    for (i=0; i<variables.Count(); i++) delete variables[i];
    for (i=0; i<contexts.Count();  i++) delete contexts[i];
  }
  void SetStop(CPosition stop) {
    this->stop = stop;
  }
  void AddVariable(CStr name, CPosition start, CPosition stop) {
    variables.Add(new CVariable(name,start,stop,false));
  }
  void AddParameter(CStr name, CPosition start, CPosition stop) {
    variables.Add(new CVariable(name,start,stop,true));
  }
  void AddReference(CStr name) {
    references.Add(name);
  }
  void AddCall(CStr name) {
    calls.Add(name);
  }
// SIMULACE BEHU
  void UseVariable(CStr name) {
    CVariable *found = NULL;
    for (int i=0; i<variables.Count(); i++) {
      if (variables[i]->name==name) {
        found = variables[i];
        break;
      }
    }
    if (found)
      found->used = true;
    else if (parent)
      parent->UseVariable(name);
    else
      printf(CStr("warning: variable ")+name+" not declared\n");
  }
  void CallFunction(CStr name) {
    CContext *found = NULL;
    for (int i=0; i<contexts.Count(); i++) {
      if (contexts[i]->name==name) {
        found = contexts[i];
        break;
      }
    }
    if (found)
      found->Execute();
    else if (parent)
      parent->CallFunction(name);
    else
      printf(CStr("warning: function ")+name+" not declared\n");
  }
  void Execute() {
    if (executed) return;
    executed = true;
    int i;
    for (i=0; i<references.Count(); i++) UseVariable(references[i]);
    for (i=0; i<calls.Count(); i++) CallFunction(calls[i]);
    for (i=0; i<contexts.Count(); i++) contexts[i]->Execute();
  }
// ODSTRANENI
  void Clean(CFileBuffer &buffer) {
    int i;
    if (!executed) {
      if (name!="") {
        printf("unused function: %s from (%d,%d) to (%d,%d)\n",name.Ptr(), start.line, start.character, stop.line, stop.character);
        buffer.Clean(start, stop);
      }
    }
    else {
      for (i=0; i<variables.Count(); i++) {
        const CVariable &var = *variables[i];
        if (!var.used && !var.protect) {
          printf("unused variable: %s from (%d,%d) to (%d,%d)\n",var.name.Ptr(), var.start.line, var.start.character, var.stop.line, var.stop.character);
          buffer.Clean(var.start, var.stop);
        }
      }
    }
    for (i=0; i<contexts.Count(); i++) contexts[i]->Clean(buffer);
  }
// VYPIS STAVU STROMU
  void Dump(CStr indent = "") {
    int i;
    printf(indent+CStr("CONTEXT ")+name+CStr(" ")+(executed ? "EXECUTED" : "NOT EXECUTED")+CStr(" {\n"));
    printf(indent+"  VARIABLES: ");
    for (i=0; i<variables.Count(); i++) {
      printf("%s-%s ",variables[i]->name.Ptr(),variables[i]->used ? "USED" : "UNUSED");
    }

    printf("\n");
    printf(indent+"  REFERENCES: ");
    for (i=0; i<references.Count(); i++) {
      printf("%s ",references[i].Ptr());
    }
    printf("\n");

    printf(indent+"  CALLS: ",indent+"  ");
    for (i=0; i<calls.Count(); i++) {
      printf("%s ",calls[i].Ptr());
    }
    printf("\n");

    for (i=0; i<contexts.Count(); i++) {
      contexts[i]->Dump(indent+"  ");
    }

    printf(indent+"}\n");
  }
};

class CSyntaxAnalyzer {
private:
  CLexicalAnalyzer lex;
  void Error(const char *msg) {
    printf("Syntax error at line %d, character %d: %s\n", lex.Line()+1, lex.Character()+1, msg);
    exit(1);
  }
  void ZbFnTyp() {
    switch (lex.Next()) {
      case IDENT:
        lex.Read(IDENT);
        break;
      case RPAR:
        break;
      default:
        Error("Expected IDENT, RPAR");
    }
  }
  void FnTyp() {
    CContext *dummy;
    switch (lex.Next()) {
      case LPAR:
        lex.Read(LPAR);
        Ref();
        ZbFnTyp();
        lex.Read(RPAR);
        lex.Read(LPAR);
        dummy = new CContext();
        Parametry(dummy);
        delete dummy;
        lex.Read(RPAR);
        break;
      default:
        break;
    }
  }
  void StructBody() {
    switch (lex.Next()) {
      case BEGIN:
        lex.SkipPairs(BEGIN,END);
        break;
      case AND:
      case ASTERISK:
      case IDENT:
        break;
      default:
        Error("Expected BEGIN, IDENT, REF, ASTERISK");
    }
  }
  void TypIdent() {
    switch (lex.Next()) {
      case IDENT:
        lex.Read(IDENT);
        break;
      case BEGIN:
        break;
      default:
        Error("Expected IDENT, BEGIN");
    }
  }
  void ZbTyp() {
    switch (lex.Next()) {
      case IDENT:
        lex.Read(IDENT);
        FnTyp();
        break;
      case ENUM:
        lex.Read(ENUM);
        TypIdent();
        StructBody();
        break;
      case STRUCT:
        lex.Read(STRUCT);
        TypIdent();
        StructBody();
        break;
      case CLASS:
        lex.Read(CLASS);
        TypIdent();
        StructBody();
        break;
      default:
        Error("Expected IDENT, ENUM, STRUCT, CLASS");
    }
  }
  void Typ() {
    switch (lex.Next()) {
      case CONST:
        lex.Read(CONST);
        Typ();
        break;
      case STATIC:
        lex.Read(STATIC);
        Typ();
        break;
      case INLINE:
        lex.Read(INLINE);
        Typ();
        break;
      case EXTERN:
        lex.Read(EXTERN);
        Typ();
        break;
      case UNSIGNED:
        lex.Read(UNSIGNED);
        Typ();
        break;
      case IDENT:
      case ENUM:
      case STRUCT:
      case CLASS:
        ZbTyp();
        break;
      case SEMICOLON:
      case COMMA:
      case RPAR:
        break;
      default:
        Error("Expected IDENT, ENUM, CONST, STATIC, INLINE, EXTERN, UNSIGNED, STRUCT, CLASS, SEMICOLON, COMMA, RPAR");
    }
  }
  void ExprIdent(CContext *context, CStr ident) {
    switch (lex.Next()) {
      case LPAR:
        context->AddCall(ident); // VOLANI FUNKCE
        break;
      default:
        context->AddReference(ident); // REFERENCE PROMENNE
        break;
    }
  }
  void Expression(CContext *context) {
    int par = 0;
    int bracket = 0;
    int iif = 0;
    int block = 0;
    while (true) {
      switch(lex.Next()) {
        case LPAR:      
          par++;
          lex.Read(LPAR);
          break;
        case RPAR:
          par--;
          if (par<0) return;
          lex.Read(RPAR);
          break;
        case LBRACKET:
          bracket++;
          lex.Read(LBRACKET);
          break;
        case RBRACKET:
          bracket--;
          if (bracket<0) return;
          lex.Read(RBRACKET);
          break;
        case BEGIN:
          block++;
          lex.Read(BEGIN);
          break;
        case END:
          block--;
          if (block<0) return;
          lex.Read(END);
          break;
        case QUESTION:
          iif++;
          lex.Read(QUESTION);
          break;
        case COLON:
          iif--;
          if (iif<0) return;
          lex.Read(COLON);
          break;
        case IDENT:
          lex.Read(IDENT);
          ExprIdent(context, lex.LastContent());
          break;
        case SEMICOLON:
        case COMMA:
          if (par>0 || bracket>0 || iif>0 || block>0) {
            lex.Read(lex.Next());
            break;
          }
          return;
        default:
          lex.Read(lex.Next());
      }
    }
  }
  void For(CContext *context) {
    lex.Read(FOR);
    lex.Read(LPAR);
    Expression(context);
    lex.Read(SEMICOLON);
    Expression(context);
    lex.Read(SEMICOLON);
    Expression(context);
    lex.Read(RPAR);
    Command(context);
  }
  void While(CContext *context) {
    lex.Read(WHILE);
    lex.Read(LPAR);
    Expression(context);
    lex.Read(RPAR);
    Command(context);
  }
  void Do(CContext *context) {
    lex.Read(DO);
    Command(context);
    lex.Read(WHILE);
    lex.Read(LPAR);
    Expression(context);
    lex.Read(RPAR);
  }
  void Else(CContext *context) {
    switch (lex.Next()) {
      case ELSE:
        lex.Read(ELSE);
        Command(context);
        break;
      default:
        break;
    }
  }
  void If(CContext *context) {
    lex.Read(IF);
    lex.Read(LPAR);
    Expression(context);
    lex.Read(RPAR);
    Command(context);
    Else(context);
  }
  void CaseBody(CContext *context) {
    switch(lex.Next()) {
      case CASE:
      case END:
      case DEFAULT:
        break;
      default:
        Command(context);
        CaseBody(context);
        break;
    }
  }
  void Case(CContext *context) {
    switch(lex.Next()) {
      case CASE:
        lex.Read(CASE);
        Expression(context);
        lex.Read(COLON);
        CaseBody(context);
        Case(context);
        break;
      case DEFAULT:
        lex.Read(DEFAULT);
        lex.Read(COLON);
        CaseBody(context);
        Case(context);
      case END:
        break;
      default:
        Error("Expected CASE, DEFAULT, END");
    }
  }
  void Switch(CContext *context) {
    lex.Read(SWITCH);
    lex.Read(LPAR);
    Expression(context);
    lex.Read(RPAR);
    lex.Read(BEGIN);
    Case(context);
    lex.Read(END);
  }
  void Return(CContext *context) {
    switch (lex.Next()) {
      case SEMICOLON:
        break;
      default:
        Expression(context);
        break;
    }
  }
  void Assignment(CContext *context, CStr lvalue) {
    context->AddReference(lvalue); // REFERENCE PROMENNE
    Expression(context);
    lex.Read(SEMICOLON);
  }
  void CallZbParam(CContext *context) {
    switch (lex.Next()) {
      case COMMA:
        lex.Read(COMMA);
        Expression(context);
        CallZbParam(context);
        break;
      case RPAR:
        break;
      default:
        Error("Expected COMMA, RPAR");
    }
  }
  void CallParam(CContext *context) {
    switch (lex.Next()) {
      case RPAR:
        break;
      default:
        Expression(context);
        CallZbParam(context);
        break;
    }
  }
  void Call(CContext *context, CStr ident) {
    context->AddCall(ident); // VOLANI FUNKCE
    lex.Read(LPAR);
    CallParam(context);
    lex.Read(RPAR);
    lex.Read(SEMICOLON);
  }
  void CommandId(CContext *context, CStr ident) {
    CPosition start = lex.Position();
    switch (lex.Next()) {
      case IDENT:
      case AND:
      case ASTERISK:
        Ref();
        lex.Read(IDENT);
        Deklarace(context,lex.LastContent(),start);
        lex.Read(SEMICOLON);
        break;
      case DOT:
        lex.Read(DOT);
        lex.Read(IDENT);
        CommandId(context,ident);
        break;
      case LPAR:
        Call(context, ident);
        break;
      case ASSIGNMENT:
        lex.Read(ASSIGNMENT);
        Assignment(context, ident);
        break;
      case PLUSASSIGNMENT:
        lex.Read(PLUSASSIGNMENT);
        Assignment(context, ident);
        break;
      case MINUSASSIGNMENT:
        lex.Read(MINUSASSIGNMENT);
        Assignment(context, ident);
        break;
      case MULTASSIGNMENT:
        lex.Read(MULTASSIGNMENT);
        Assignment(context, ident);
        break;
      case DIVASSIGNMENT:
        lex.Read(DIVASSIGNMENT);
        Assignment(context, ident);
        break;
      case ANDASSIGNMENT:
        lex.Read(ANDASSIGNMENT);
        Assignment(context, ident);
        break;
      case ORASSIGNMENT:
        lex.Read(ORASSIGNMENT);
        Assignment(context, ident);
        break;
      default:
        Expression(context);
        lex.Read(SEMICOLON);
        break;
    }
  }
  void Command(CContext *context) {
    CPosition start;
    switch (lex.Next()) {
      case IDENT:
        lex.Read(IDENT);
        CommandId(context, lex.LastContent());
        break;
      case CONST :
      case STATIC :
      case INLINE :
      case EXTERN :
      case UNSIGNED :
      case STRUCT :
      case CLASS :
      case ENUM :
        Typ();
        start = lex.Position();
        Ref();
        lex.Read(IDENT);
        Deklarace(context, lex.LastContent(), start);
        lex.Read(SEMICOLON);
        break;
      case TYPEDEF :
        lex.Read(TYPEDEF);
        Typ();
        Typedef();
        break;
      case BEGIN:
        Blok(context);
        break;
      case FOR:
        For(context);
        break;
      case WHILE:
        While(context);
        break;
      case DO:
        Do(context);
        break;
      case IF:
        If(context);
        break;
      case SWITCH:
        Switch(context);
        break;
      case RETURN:
        lex.Read(RETURN);
        Return(context);
        lex.Read(SEMICOLON);
        break;
      case BREAK:
        lex.Read(BREAK);
        lex.Read(SEMICOLON);
        break;
      case CONTINUE:
        lex.Read(CONTINUE);
        lex.Read(SEMICOLON);
        break;
      case SEMICOLON:
        lex.Read(SEMICOLON);
        break;
      default:
        Error("XXXXX");
    }
  }
  void ZbBlok(CContext *context) {
    switch (lex.Next()) {
      case END:
        break;
      default:
        Command(context);
        ZbBlok(context);
        break;
    }
  }
  void Blok(CContext *context) {
    CContext *blok = new CContext(context);
    lex.Read(BEGIN);
    ZbBlok(blok);
    lex.Read(END);
  }
  void ZbOnePar(CContext *context, CPosition start) {
    CPosition stop;
    switch (lex.Next()) {
      case IDENT:
        lex.Read(IDENT);
        Index(context);
        Construct(context);
        stop = lex.Position();
        context->AddParameter(lex.LastContent(), start, stop); // DEKLARACE PROMENNE - PARAMETR
        break;
      case RPAR:
      case COMMA:
        break;
      default:
        Error("Expected IDENT, RPAR, COMMA");
    }
  }
  void OnePar(CContext *context, CPosition start) {
    switch (lex.Next()) {
      case AND:
      case ASTERISK:
      case IDENT:
        Ref();
        ZbOnePar(context, start);
        break;
      case RPAR:
      case COMMA:
        break;
      default:
        Error("Expected AND, ASTERISK, IDENT, RPAR, COMMA");
    }
  }
  void ZbPar(CContext *context) {
    CPosition start;
    switch (lex.Next()) {
      case COMMA:
        start = lex.Position();
        lex.Read(COMMA);
        Typ();
        OnePar(context, start);
        ZbPar(context);
        break;
      case RPAR:
        break;
      default:
        Error("Expected COMMA, RPAR");
    }
  }
  void Parametry(CContext *context) {
    CPosition start = lex.Position();
    switch (lex.Next()) {
      case IDENT:
      case CONST:
      case UNSIGNED:
      case STATIC:
      case INLINE:
      case EXTERN:
      case STRUCT:
      case CLASS:
      case ENUM:
        Typ();
        OnePar(context, start);
        ZbPar(context);
        break;
      case RPAR:
        break;
      default:
        Error("Expected IDENT, CONST, UNSIGNED, EXTERN, STATIC, INLINE, RPAR");
    }
  }
  void Index(CContext *context) {
    switch (lex.Next()) {
      case LBRACKET:
        lex.Read(LBRACKET);
        Expression(context);
        lex.Read(RBRACKET);
        break;
      case COMMA:
      case SEMICOLON:
      case ASSIGNMENT:
      case LPAR:
      case RPAR:
        break;
      default:
        Error("Expected COMMA, SEMICOLON, LBRACKET, ASSIGNMENT, LPAR, RPAR");
    }
  }
  void Ref() {
    switch (lex.Next()) {
      case ASTERISK:
        lex.Read(ASTERISK);
        Ref();
        break;
      case AND:
        lex.Read(AND);
        Ref();
        break;
      case IDENT:
      case COMMA:
      case SEMICOLON:
      case RPAR:
        break;
      default:
        Error("Expected ASTERISK, AND, IDENT, RPAR, COMMA, SEMICOLON");
    }
  }
  void Construct(CContext *context) {
    switch (lex.Next()) {
      case LPAR:
        lex.Read(LPAR);
        CallParam(context);
        lex.Read(RPAR);
        break;
      case COMMA:
      case ASSIGNMENT:
      case SEMICOLON:
      case RPAR:
        break;
      default:
        Error("Expected COMMA, SEMICOLON, ASSIGNMENT, LPAR, RPAR");
    }
  }
  void FuncBody(CContext *context, CStr ident) {
    switch (lex.Next()) {
      case BEGIN:
        Blok(context);
        break;
      case SEMICOLON:
        printf("ANULOVAT FUNKCI, JEN FORWARD\n");
//        context->parent->Remove(context);
        lex.Read(SEMICOLON);
        break;
      default:
        Error("Expected BEGIN, SEMICOLON");
    }
  }
  void Funkce(CContext *context, CStr ident, CPosition start) {
    CContext *function = new CContext(context, ident, start); // DEKLARACE FUNKCE
    lex.Read(LPAR);
    Parametry(function);
    lex.Read(RPAR);
    FuncBody(function, ident);
    function->SetStop(lex.Position());
  }
  void DeklAssign(CContext *context) {
    switch (lex.Next()) {
      case ASSIGNMENT:
        lex.Read(ASSIGNMENT);
        Expression(context);
        break;
      case COMMA:
      case SEMICOLON:
        break;
      default:
        Error("Expected ASSIGNMENT, COMMA, SEMICOLON");
    }
  }
  void ZbDekl(CContext *context, CStr old_ident, CPosition old_start, CPosition old_stop) {
    CPosition new_stop, new_start = lex.Position();
    CStr new_ident;
    switch (lex.Next()) {
      case COMMA:
        lex.Read(COMMA);
        if (old_stop.line==-1) old_stop = lex.Position();
        context->AddVariable(old_ident,old_start,old_stop); // DEKLARACE PROMENNE
        Ref();
        lex.Read(IDENT);
        new_ident = lex.LastContent();
        Index(context);
        Construct(context);
        DeklAssign(context);
        ZbDekl(context,new_ident,new_start,lex.Position());
        break;
      case SEMICOLON:
        context->AddVariable(old_ident,old_start,lex.Position()); // DEKLARACE PROMENNE
        break;
      default:
        Error("Expected COMMA, SEMICOLON");
    }
  }
  void Deklarace(CContext *context, CStr ident, CPosition start) {
    Index(context);
    Construct(context);
    DeklAssign(context);
    ZbDekl(context,ident,start,CPosition());
  }
  void Typedef() {
    CContext *dummy = new CContext();
    switch (lex.Next()) {
      case IDENT:
      case AND:
      case ASTERISK:
        Ref();
        lex.Read(IDENT);
        Index(dummy);
        lex.Read(SEMICOLON);
        break;
      case SEMICOLON:
        lex.Read(SEMICOLON);
        break;
      default:
        Error("Expected IDENT, AND, ASTERISK, SEMICOLON");
    }
    delete dummy;
  }
  void ZbElement(CContext *context, CStr ident, CPosition startfunc, CPosition startvar) {
    switch (lex.Next()) {
      case COMMA:
      case SEMICOLON:
      case LBRACKET:
      case ASSIGNMENT:
        Deklarace(context, ident, startvar);
        lex.Read(SEMICOLON);
        break;
      case LPAR:
        Funkce(context, ident, startfunc);
        break;
      default:
        Error("Expected COMMA, SEMICOLON, LPAR, ASSIGNMENT");
    }
  }
  void ElementTyp(CContext *context, CPosition startfunc, CPosition startvar) {
    switch (lex.Next()) {
      case IDENT:
        lex.Read(IDENT);
        ZbElement(context, lex.LastContent(), startfunc, startvar);
        break;
      case SEMICOLON:
        lex.Read(SEMICOLON);
        break;
      default:
        Error("Expected IDENT, SEMICOLON");
    }
  }
  void Element(CContext *context) {
    CPosition startvar, startfunc = lex.Position();
    switch (lex.Next()) {
      case IDENT :
      case CONST :
      case STATIC :
      case INLINE :
      case EXTERN :
      case UNSIGNED :
      case STRUCT :
      case CLASS :
      case ENUM :
        Typ();
        startvar = lex.Position();
        Ref();
        ElementTyp(context, startfunc, startvar);
        break;
      case TYPEDEF :
        lex.Read(TYPEDEF);
        Typ();
        Typedef();
        break;
      default:
        Error("Expected IDENT, TYPEDEF, CONST, STATIC, INLINE, EXTERN, UNSIGNED, CLASS, STRUCT, ENUM");
    }
  }
  void Program(CContext *context) {
    switch (lex.Next()) {
      case IDENT :
      case TYPEDEF :
      case CONST :
      case UNSIGNED :
      case STATIC :
      case INLINE :
      case EXTERN :
      case CLASS :
      case STRUCT :
      case ENUM :
        Element(context);
        Program(context);
        break;
      case EOI:
        break;
      default:
        Error("Expected IDENT, TYPEDEF, CONST, UNSIGNED, STATIC, INLINE, CLASS, STRUCT, ENUM, EOI");
    }
  }
public:
  CSyntaxAnalyzer(FILE *input) : lex(input) {
    lex.Ignore(DIRECTIVE);
    lex.Ignore(COMMENT);
    lex.Ignore(_CDECL);
    lex.Init();
  }
  void Start(FILE *output) {
  // CREATE ROOT CONTEXT
    CContext *root = new CContext();
  // ANALYSE  
    Program(root);
  // SIMULATE EXECUTION
    root->executed = true;
    root->CallFunction("main");
  // DUMP
    root->Dump();
  // FIND AND CLEAN UNUSED
    root->Clean(lex.buffer);
  // OUTPUT TO FILE
    lex.buffer.Output(output);
  // FREE MEMORY
    delete root;
  }
};

void ParseLexicalSymbols() {
  CLexicalAnalyzer lex(stdin);
  Token token;
  while ((token = lex.Next()) != EOI) {
    lex.Read(token);
  }
}

void ParseSyntax(const char *in_name, const char *out_name) {
  FILE *in = fopen(in_name, "r");
  FILE *out = fopen(out_name, "w");
  CSyntaxAnalyzer syntax(in);
  syntax.Start(out);
  fclose(in);
  fclose(out);
}

const char *usage = "Usage: remove input.cpp output.cpp\n";

int main(int argc, char *argv[],char *envp[]) {
  if (1+EOI!=TokenCount) return 2;
  if (argc<3) {
    printf(usage);
    return 1;
  }
  ParseSyntax(argv[1],argv[2]);
  return 0;
}
